Attribute VB_Name = "HuntERR30"
'========================================================================================
'                               HuntERR
'                   Error Handling and Reporting Library
'                    from URFIN JUS (www.urfinjus.net)
'                  Copyright 2001. All rights reserved.
'version 3.00, 07/15/2001
'=========================================================================================
'Conditional compilation constants
'  H_NOCOMPLUS = 1     -- No COM+ support
'  H_SYSH = 1          -- Include support for system exceptions handling
'  H_NOENUMS = 1       -- Exclude declarations of ENUM_ERRMAP enumeration.

'Public members =========================================================================
'
'Public Enum ENUM_ERRMAP
'Public Enum ENUM_ERROR_ACTION

'Public Function ErrorIn(ByVal MethodHeader As String, _
'                        Optional ByVal arrArgs, _
'                        Optional ByVal ErrorAction As Long = EA_DEFAULT, _
'                        Optional ByVal DbObject As Object, _
'                        Optional ByVal EnvVarNames As String, _
'                        Optional ByVal arrEnvVars, _
'                        Optional ByVal TransControlObject As Object) As String
'Public Sub Check(ByVal Cond As Boolean, _
'                 ByVal ErrNumber As Long, ByVal ErrDescr As String)
'
'Public Sub ErrPreserve()
'Public Sub ErrRestore()
'Public Sub ErrContinue(ByVal AnErrNum As Long, ByVal AnErrReport)
'Public Sub ErrGetFromServer(ByVal Extractor, ByVal COMServer As Object, _
'        Optional ByVal Param, Optional ByVal Comment As String)

'Public Function IsException(ByVal ErrNumber As Long) As Boolean
'Public Function InException() As Boolean
'Public Function InPropagation() As Boolean

'Public Property Get ErrReport() As String
'Public Property Get ErrReportHTML() As String
'Public Property Get ErrNumber() As Long
'Public Property Get ErrSource() As String
'Public Property Get ErrDescription() As String
'Public Property Get ErrOrigSource() As String
'Public Property Get ErrOrigDescription() As String
'Public Function ErrExtractFromReport(ByVal AReport As String, ByVal FromStr As String, ByVal TillStr As String)
'
'Public Function ErrSaveToEventLog() As Boolean
'Public Function ErrSaveToFile(Optional ByVal ErrFileName As String = "Errors.txt") As Boolean
'Public Function ErrSaveToDB(ByVal ConnectString As String, _
'                       Optional ByVal AppID As Long = 1, _
'                       Optional ByVal ProcName As String = "spErrorLogInsert") As Boolean

Option Explicit

#If H_NOENUMS = 1 Then
    'ENUM_ERRMAP is declared in other module (we recommend public COM class)
#Else
'Error map enum. Defines error ranges for errors and exceptions
'Define your custom errors starting with ERRMAP_APP_FIRST + 1.
'EXC_GENERAL and ERR_GENERAL may be used by your application if specific error number is not needed.
'Error numbers (vbObjectError + [1..4095])  are used by OLE DB;
'See support.microsoft.com/support/kb/articles/Q168/3/54.ASP
Public Enum ENUM_ERRMAP
    ERRMAP_FIRST = vbObjectError + 4096 'vbObjectError = $H80040000 = -2147221504
    ERRMAP_RESERVED_FIRST = ERRMAP_FIRST            'Errors reserved for HuntERR and UJ apps.
       ERR_SYSEXCEPTION                        'System exception like access violation
    ERRMAP_RESERVED_LAST = ERRMAP_RESERVED_FIRST + 100
    ERRMAP_EXC_FIRST = ERRMAP_RESERVED_LAST + 1 'Exceptions - reraised by ErrorIn
        EXC_GENERAL = ERRMAP_EXC_FIRST
    ERRMAP_EXC_LAST = ERRMAP_EXC_FIRST + 1000
    ERRMAP_APP_FIRST
        ERR_GENERAL = ERRMAP_APP_FIRST ' = vbObjectError + 4096+1+1000+1
        'Application errors here
End Enum
#End If 'H_NOENUMS = 1 ... Else ...

'Flags controlling actions of ErrorIn, through ErrorAction parameter
Public Enum ENUM_ERROR_ACTION
    EA_RERAISE = 1   'Reraise error
    EA_ADVANCED = 2 'Add trace information to error description
    EA_SET_ABORT = 4 'Call SetAbort on current object's context.
    EA_DISABLE_COMMIT = 8 'Call DisableCommit on current objects' context. Recommended.
    EA_ROLLBACK = 16 'Call Connection.Rollback
    EA_WEBINFO = 32 'Add web request information
    EA_DEFAULT = EA_ADVANCED + EA_RERAISE + EA_DISABLE_COMMIT 'Default value for ErrorIn
End Enum

Private mErrNumber As Long, mErrSource As String, mErrDescr As String, mErl As Long
Private mErrReport As String, mErrAPI As Long
Private mErrPreserved As Boolean 'Set by SaveErr, cleared by ErrorIn at the end of processing
Private mExtBuffer As String 'Errors from COMServers, added by ErrGetFromServer
Private Const SRC_ERRORIN = "HuntERR.ErrorIn"

'API declarations
Private Declare Function GetComputerNameAPI Lib "kernel32" Alias "GetComputerNameA" _
    (ByVal lpBuffer As String, nSize As Long) As Long
Private Declare Function FormatMessageAPI Lib "kernel32" Alias "FormatMessageA" _
    (ByVal dwFlags As Long, lpSource As Any, ByVal dwMessageId As Long, _
    ByVal dwLanguageId As Long, ByVal lpBuffer As String, ByVal nSize As Long, _
    Arguments As Long) As Long

'The main super-function. Should be called in error handling blocks
Public Function ErrorIn(ByVal MethodHeader As String, _
                        Optional ByVal arrArgs, _
                        Optional ByVal ErrorAction As Long = EA_DEFAULT, _
                        Optional ByVal DbObject As Object, _
                        Optional ByVal EnvVarNames As String, _
                        Optional ByVal arrEnvVars, _
                        Optional ByVal TransControlObject As Object) As String
    Dim objConn As Object, MethodName As String, ArgNames As String
    ErrPreserve   'preserve error information in module-scope variables.
    If IsException(mErrNumber) Then
        mErrReport = ""
    ElseIf FlagSet(ErrorAction, EA_ADVANCED) Then
        Set objConn = ErrFindConnectionObject(DbObject)
        ParseMethodHeader MethodHeader, MethodName, ArgNames
        If InPropagation Then
            mErrReport = mErrDescr
            Else 'Initial processing
            ReportInit MethodName
            ReportAddAPIError
            ReportAddADOErrors objConn
        End If 'ErrNumber=ERRDB_STACK_TRACE ....
        If FlagSet(ErrorAction, EA_WEBINFO) Then ReportAddWebInfo
        ReportAddExtErrors
        ReportAddTraceInfo MethodName, ArgNames, arrArgs, EnvVarNames, arrEnvVars
        #If H_SYSH = 1 Then
            If ErrSysHandlerWasSet Then ErrSysHandlerRelease
        #End If
        mErrSource = SRC_ERRORIN
        mErrDescr = mErrReport
    End If 'IsException(.... ElseIf FlagSet(...
    ErrTerminateTrans ErrorAction, TransControlObject, objConn
    ErrorIn = mErrReport
    ErrRestore
    mErrPreserved = False
    If FlagSet(ErrorAction, EA_RERAISE) Then Err.Raise Err.Number, Err.Source, Err.Description
  End Function

Public Sub Check(ByVal Cond As Boolean, _
                 ByVal ErrNumber As Long, ByVal ErrDescr As String)
If Not Cond Then Err.Raise ErrNumber, "HuntERR.Check", ErrDescr
End Sub

'Preserves Err object properties for later use by ErrorIn
Public Sub ErrPreserve()
    If Not mErrPreserved Then
        mErrNumber = Err.Number
        mErrDescr = Err.Description
        mErrSource = Err.Source
        mErl = Erl
        mErrAPI = Err.LastDllError 'We need to do it here, GetLastError information is vulnerable
        mErrPreserved = True
    End If
End Sub

'Restores Err object properties
Public Sub ErrRestore()
    Err.Number = mErrNumber
    Err.Source = mErrSource
    Err.Description = mErrDescr
End Sub

'Continues propagation process
Public Sub ErrContinue(ByVal AnErrNum As Long, ByVal AnErrReport)
    Err.Raise AnErrNum, SRC_ERRORIN, AnErrReport
End Sub

'Retrieves error information from custom COM object (server).
'Uses extractor class provided by application.
'Extactor object should have Extract method returning formatted error information.
Public Sub ErrGetFromServer(ByVal Extractor, ByVal COMServer As Object, _
        Optional ByVal Param, Optional ByVal Comment As String)
    Dim objExtr As Object, sMsg As String, sHdr As String, sFtr As String
    ErrPreserve
    On Error GoTo errHandler
    sHdr = "  COM Server Errors: Server=" & ErrVarToString(COMServer) & _
        " Extractor=" & ErrVarToString(Extractor) & _
        IIf(Comment = "", "", "  [" & Comment & "]")
    sFtr = "  End COM Server Errors"
    If IsObject(Extractor) Then Set objExtr = Extractor Else Set objExtr = CreateObject(CStr(Extractor))
    ErrRestore 'Set Err object with original error information, in case if extractor needs it
    sMsg = objExtr.Extract(COMServer, Param)
    If sMsg <> "" Then
        If Right$(RTrim$(sMsg), Len(vbNewLine)) <> vbNewLine Then sMsg = sMsg & vbNewLine
        mExtBuffer = mExtBuffer & sHdr & vbNewLine & sMsg & sFtr & vbNewLine
    End If
    GoTo ExitSub 'We cannot use Exit Sub - it clears Err object!
errHandler:
    If mExtBuffer <> "" Then mExtBuffer = mExtBuffer & vbNewLine
    mExtBuffer = mExtBuffer & sHdr & vbNewLine & _
        "    ErrorIn failed to extract error information: " & Err.Description
ExitSub:
    ErrRestore
End Sub

'Returns true if parameter is in range reserved for Exceptions
Public Function IsException(ByVal ErrNumber As Long) As Boolean
    IsException = (ErrNumber >= ERRMAP_EXC_FIRST) And (ErrNumber <= ERRMAP_EXC_LAST)
End Function

'Returns true if Err.Number is in range reserved for Exceptions
Public Function InException() As Boolean
    InException = IsException(ErrNumber)
End Function

'Returns true if error was raised by ErrorIn, thus propagation is in progress
Public Function InPropagation() As Boolean
    InPropagation = (ErrSource = SRC_ERRORIN)
End Function

'Returns report prepared by last call to ErrorIn, or current Err.Description
Public Property Get ErrReport() As String
    If mErrReport <> "" Then
        ErrReport = mErrReport
    ElseIf InPropagation Then
        ErrReport = Err.Description
    Else
        ErrReport = ""
    End If
End Property

'Returns HTML-formatted error report
Public Property Get ErrReportHTML() As String
    Dim sHTML As String
    sHTML = ErrReport
    sHTML = Replace(sHTML, "&", "&amp;")
    sHTML = Replace(sHTML, "<", "&lt;")
    sHTML = Replace(sHTML, ">", "&gt;")
    sHTML = Replace(sHTML, """", "&quot;")
    sHTML = Replace(sHTML, " ", "&nbsp;")
    sHTML = Replace(sHTML, vbNewLine, "<br>" & vbNewLine)
    ErrReportHTML = sHTML
End Property

'Returns error number saved by last call to ErrPreserve or ErrorIn, or curre
Public Property Get ErrNumber() As Long
    ErrNumber = mErrNumber
End Property

'Returns error source saved by last call to ErrPreserve or ErrorIn, or curre
Public Property Get ErrSource() As String
    ErrSource = mErrSource
End Property

'Returns error description saved by last call to ErrPreserve or ErrorIn, or curre
Public Property Get ErrDescription() As String
    ErrDescription = mErrDescr
End Property

'Extracts original error source from error report
Public Property Get ErrOrigSource() As String
    ErrOrigSource = ErrExtractFromReport("Source: ", vbNewLine)
End Property

'Extracts original error description from error report
Public Property Get ErrOrigDescription() As String
    Dim P As Long
    If ErrReport <> "" Then
        P = InStr(1, ErrReport, vbNewLine)
        If P = 0 Then P = Len(ErrReport) + 1
        ErrOrigDescription = Left$(ErrReport, P - 1)
    End If
End Property

'Extracts substring from error report
Public Function ErrExtractFromReport(ByVal FromStr As String, ByVal TillStr As String)
    Dim PStart As Long, PEnd As Long, Rpt As String
    Rpt = ErrReport
    If Rpt = "" Then Exit Function
    PStart = InStr(1, Rpt, FromStr) + Len(FromStr)
    PEnd = InStr(PStart, Rpt, TillStr) - 1
    If (PStart > 0) And (PEnd >= PStart) Then ErrExtractFromReport = Mid$(Rpt, PStart, PEnd - PStart + 1)
End Function

'Logs error report to system event log
'Logging is ignored from within VB IDE!
Public Function ErrSaveToEventLog() As Boolean
    On Error GoTo errHandler
    If ErrReport <> "" Then
        App.StartLogging "", vbLogToNT
        App.LogEvent ErrReport
    End If 'mErrReport...
    ErrSaveToEventLog = True
    Exit Function
errHandler:
    'nothing to do...
End Function

'Appends error report to text file
Public Function ErrSaveToFile(Optional ByVal ErrFileName As String = "Errors.txt") As Boolean
    Dim F As Long, FName As String
    On Error GoTo errHandler
    If ErrReport <> "" Then
        F = FreeFile
        FName = IIf(InStr(1, ErrFileName, "\") > 0, ErrFileName, App.Path & "\" & ErrFileName)
        If OpenErrorFile(FName, F) Then
            Print #F, ErrReport & vbNewLine & vbNewLine
            Close #F
        End If
    End If 'mErrReport...
    ErrSaveToFile = True
    Exit Function
errHandler:
    'nothing to do...
End Function

'Logs error report to database table
'Note: SQL script for creating database table and stored procedure is at the end of this file.
Public Function ErrSaveToDB(ByVal ConnectString As String, _
                       Optional ByVal AppID As Long = 1, _
                       Optional ByVal ProcName As String = "spErrorLogInsert") As Boolean
    Dim Cmd As Object, SQL As String, Rpt As String
    On Error GoTo errHandler
    If ErrReport <> "" Then
        SQL = "Exec " & ProcName & " " & AppID & ", " & "'" & Replace(ErrReport, "'", "''") & "'"
        Set Cmd = CreateObject("ADODB.Command")
        Cmd.CommandType = 1 ' adCmdText
        Cmd.CommandText = SQL
        Cmd.ActiveConnection = ConnectString
        Cmd.Execute
    End If
    ErrSaveToDB = True
    Exit Function
errHandler:
    App.StartLogging "", vbLogToNT 'Try to save message to Event Log
    App.LogEvent "Failed to save error report to database. " & vbNewLine & _
            "ConnectString = '" & ConnectString & "'"
End Function


'##################################### Private methods #########################################
'Prepares initial report
Private Sub ReportInit(ByVal MethodName As String)
Const TEMPLATE_REPORT = _
  "<$Descr$> <$nl$>" & _
  "    Time='<$Time$>' App='<$App$>:<$Ver$>' Server='<$Server$>' InTrans=<$InTrans$> <$nl$>" & _
  "    Method: <$MethodName$> <$nl$>" & _
  "    Number: <$ErrNum$> = &H<$ErrHex$> = vbObjectError <$ErrNumRel1$> = ERRMAP_APP_FIRST <$ErrNumRel2$> <$ErrReserved$><$nl$>" & _
  "    Source: <$Source$> <$nl$>" & _
  "    Description: <$Descr$><$nl$>"
    Dim Ctx As Object, InTrans As Boolean
    On Error GoTo errHandler
    mErrReport = TEMPLATE_REPORT
    ReportSet "nl", vbNewLine
    ReportSet "MethodName", MethodName
    ReportSet "Server", ErrGetComputerName
    ReportSet "Time", Format(Now, "Mm/Dd/yy Hh:Nn:Ss")
    ReportSet "App", App.EXEName
    ReportSet "Ver", ErrGetAppVersion
    Set Ctx = GetContext
    If Not Ctx Is Nothing Then InTrans = Ctx.IsInTransaction
    ReportSet "InTrans", IIf(InTrans, "Yes", "No")
    ReportSet "ErrNum", mErrNumber
    ReportSet "ErrHex", Hex$(mErrNumber)
    ReportSet "ErrNumRel1", FormatNum(mErrNumber - vbObjectError)
    ReportSet "ErrNumRel2", FormatNum(mErrNumber - ERRMAP_APP_FIRST)
    Select Case mErrNumber
        Case ERR_GENERAL: ReportSet "ErrReserved", "= ERR_GENERAL"
        Case ERR_SYSEXCEPTION: ReportSet "ErrReserved", "= ERR_SYSEXCEPTION"
        Case Else: ReportSet "ErrReserved", ""
    End Select
    ReportSet "Source", mErrSource
    ReportSet "Descr", mErrDescr
    Exit Sub
errHandler:
    'Nothing to do...
End Sub

Private Sub ReportSet(ByVal Tag As String, ByVal Value As String)
    mErrReport = Replace(mErrReport, "<$" & Tag & "$>", Value)
End Sub


Private Sub ReportAdd(ByVal Info As String)
    mErrReport = mErrReport & Info & vbNewLine
End Sub

Private Sub ReportAddAPIError()
    On Error GoTo errHandler
    '203 is "System cannot find the environment", not so much meaningful
    If (mErrAPI <> 0) And (mErrAPI <> 203) Then
        ReportAdd "    API Error: (" & mErrAPI & ") " & FormatMessage(mErrAPI)
    End If
    Exit Sub
errHandler:
    'Nothing to do...
End Sub

Private Sub ReportAddExtErrors()
    On Error GoTo errHandler
    If mExtBuffer <> "" Then
        mErrReport = mErrReport & mExtBuffer 'Ext buffer must already have vbNewLine at the end.
        mExtBuffer = ""
    End If
    Exit Sub
errHandler:
    'Nothing to do...
End Sub

Private Function FormatMessage(ByVal ErrNum As String) As String
Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
    Dim strBuffer As String * 512, strMsg As String
    On Error GoTo errHandler
    FormatMessageAPI FORMAT_MESSAGE_FROM_SYSTEM, Null, ErrNum, 0, strBuffer, 512, 0
    strMsg = strBuffer
    'Strange but necessary manipulations
    strMsg = Replace(strMsg, vbNewLine, "")
    strMsg = Replace(strMsg, Chr(0), "")
    FormatMessage = strMsg
    Exit Function
errHandler:
    'Nothing to do...
End Function

Private Sub ErrTerminateTrans(ByVal ErrorAction As Long, _
                                 ByVal TransControlObject As Object, _
                                 ByVal Conn As Object)
    Dim Ctx As Object
    On Error GoTo errHandler
    If Not TransControlObject Is Nothing Then
        On Error Resume Next
        TransControlObject.SetAbort
    ElseIf FlagSet(ErrorAction, EA_SET_ABORT) Then
           Set Ctx = GetContext
           If Not Ctx Is Nothing Then Ctx.SetAbort
    ElseIf FlagSet(ErrorAction, EA_DISABLE_COMMIT) Then
           Set Ctx = GetContext
           If Not Ctx Is Nothing Then Ctx.SetAbort
    ElseIf (Not Conn Is Nothing) And FlagSet(ErrorAction, EA_ROLLBACK) Then
        If Conn.State = 1 Then  '1=adStateOpen
            On Error Resume Next 'In case if there is no transaction
            Conn.RollbackTrans
        End If 'Conn.State
    End If 'Not TransControlObject ....
    Exit Sub
errHandler:
    'Nothing to do...
End Sub

Private Function ErrFindConnectionObject(ByVal DbObject As Object) As Object
    On Error GoTo errHandler
    Set ErrFindConnectionObject = Nothing
    If DbObject Is Nothing Then Exit Function
    Select Case TypeName(DbObject)
        Case "Connection":  Set ErrFindConnectionObject = DbObject
        Case "Command", "Recordset":     Set ErrFindConnectionObject = DbObject.ActiveConnection
        Case Else:
            Set ErrFindConnectionObject = DbObject.Connection 'Custom class, try to get its Connection property
    End Select
    Exit Function
errHandler:
    'Nothing to do
End Function

Private Sub ReportAddADOErrors(ByVal objConn As Object)
    Dim E As Object
    On Error GoTo errHandler
    If objConn Is Nothing Then Exit Sub
    If objConn.Errors.Count = 0 Then Exit Sub
    ReportAdd "  ADO Errors:"
    For Each E In objConn.Errors
        ReportAdd "    " & E.Description
    Next E
    Exit Sub
errHandler:
    'Nothing to do: failed for whatever reason, so no ADO errors
End Sub

'Reads information from IIS request object
Private Sub ReportAddWebInfo()
Const TEMPLATE_WEBINFO = _
  "<$nl$>" & _
  "  <$WebHeader$> <$nl$>" & _
  "    RequestMethod='<$RequestMethod$>'<$nl$>" & _
  "    QueryString: '<$WebServer$><$URL$><$QS$>' <$nl$>" & _
  "    FormData:    '<$FormData$>' <$nl$>" & _
  "    Cookies:     '<$Cookies$>'  <$nl$>" & _
  "  End WEB Info "
Const STR_WEBHEADER = "WEB Info:" 'Web header is used to detect if Web information is already added.
    Dim objContext As Object, IISRequestObj As Object
    Dim strReqMethod As String, strCookies As String, strServer As String
    On Error GoTo errHandler
    'Check if Web Request Info is already there
    If InStr(1, mErrReport, STR_WEBHEADER) > 0 Then Exit Sub
    'Try to get Request object through ObjectContext
    Set objContext = GetContext
    If IsEmpty(objContext) Or (objContext Is Nothing) Then Exit Sub
    Set IISRequestObj = objContext("Request")
    'Request object retrieved, add web request info to the report
    ReportAdd TEMPLATE_WEBINFO
    With IISRequestObj
        ReportSet "WebHeader", STR_WEBHEADER
        ReportSet "WebServer", .ServerVariables("SERVER_NAME")
        ReportSet "RequestMethod", .ServerVariables("REQUEST_METHOD")
        ReportSet "URL", .ServerVariables("URL")
        ReportSet "QS", IIf(.QueryString = "", "", "?" & .QueryString)
        ReportSet "FormData", CStr(.Form)
        ReportSet "Cookies", .Cookies
        ReportSet "nl", vbNewLine
    End With
    Exit Sub
errHandler:
    'Nothing to do
End Sub

Private Sub ParseMethodHeader(ByVal MethodHeader As String, ByRef MethodName As String, _
                ByRef ArgNames As String)
    On Error GoTo errHandler
    Dim arrBuf() As String
    MethodName = ""
    ArgNames = ""
    arrBuf = Split(MethodHeader, "(")
    If UBound(arrBuf) >= 0 Then MethodName = arrBuf(0)
    If UBound(arrBuf) > 0 Then ArgNames = Left$(arrBuf(1), Len(arrBuf(1)) - 1)    'get rid of ")"
    Exit Sub
errHandler:
    'Nothing to do
End Sub

Private Sub ReportAddTraceInfo(ByVal MethodName As String, _
                                 ByVal ArgNames As String, _
                                 ByVal arrArgs, _
                                 ByVal EnvVarNames As String, _
                                 ByVal arrEnvVars)
    On Error GoTo errHandler
    ReportAdd "  Trace: " & MethodName & "(" & ErrCreateNameValueList(ArgNames, arrArgs) & ")" & _
        IIf(mErl = 0, "", " Line=" & mErl)
    If EnvVarNames <> "" Then
         ReportAdd "    Env: " & ErrCreateNameValueList(EnvVarNames, arrEnvVars)
    End If
    Exit Sub
errHandler:
   'Nothing to do
End Sub

Private Function ErrCreateNameValueList(ByVal strNames As String, ByVal arrValues) As String
    Dim arrNames() As String, i As Long, strList As String, strValue As String, strNameValue As String
    On Error GoTo errHandler
    'arrValues maybe array of values, or a single value.
    If Not IsArray(arrValues) Then arrValues = Array(arrValues)
    arrNames = Split(strNames, ",")
    For i = 0 To UBound(arrValues)
        strValue = ErrVarToString(arrValues(i))
        If i <= UBound(arrNames) Then
            strNameValue = arrNames(i) & "=" & strValue
            Else
            strNameValue = strValue
        End If 'i<=....
        If strList <> "" Then strList = strList & ", "
        strList = strList & strNameValue
    Next i
    ErrCreateNameValueList = strList
    Exit Function
errHandler:
    'Nothing to do: failed for whatever reason
End Function

 'Utilities =========================================================================
Private Function FlagSet(ByVal Value As Long, ByVal Flag As Long) As Boolean
    FlagSet = ((Value And Flag) <> 0)
End Function

Private Function ErrGetComputerName() As String
    Dim sBuffer As String * 255, lLen As Long
    lLen = Len(sBuffer)
    If CBool(GetComputerNameAPI(sBuffer, lLen)) Then ErrGetComputerName = Left$(sBuffer, lLen)
End Function

Private Function ErrGetAppVersion() As String
    ErrGetAppVersion = App.Major & "." & App.Minor & "." & App.Revision
End Function

Private Function ErrVarToString(ByVal V) As String
  Dim L As Long, U As Long
  On Error GoTo errHandler
  If IsArray(V) Then
        ErrVarToString = "{Array}"
    Else 'If IsArray(...
    Select Case VarType(V)
        Case vbInteger, vbLong, vbByte, _
             vbSingle, vbDouble, vbCurrency, _
             vbBoolean, vbDecimal:
                      ErrVarToString = CStr(V)
        Case vbDate:      ErrVarToString = "'" & CStr(V) & "'"
        Case vbError:     ErrVarToString = "" 'Missing arg falls here
        Case vbEmpty:     ErrVarToString = "{Empty}"
        Case vbNull:      ErrVarToString = "{Null}"
        Case vbString:    ErrVarToString = "'" & V & "'"
        Case vbObject:    ErrVarToString = "{" & TypeName(V) & "}"
        Case Else:        ErrVarToString = "{?}"
        End Select
    End If 'IsArray...
  Exit Function
errHandler:
  ErrVarToString = "{?}"
  End Function
 
Private Function FormatNum(ByVal L As Long) As String
    FormatNum = IIf(L >= 0, "+ " & L, "- " & Abs(L))
End Function

'File may be opened by other component. Keep trying for 100 ms to open.
Private Function OpenErrorFile(ByVal FileName As String, ByRef F As Long) As Boolean
    Const MS100 = 1# / 24# / 60# / 60# / 10
    Dim StartTime As Date, Opened As Boolean
    If TryOpenErrorFile(FileName, F) Then OpenErrorFile = True: Exit Function
    StartTime = Now
    Do
      DoEvents
      Opened = TryOpenErrorFile(FileName, F)
    Loop Until Opened Or (Now - StartTime) > MS100
    OpenErrorFile = Opened
End Function

Private Function TryOpenErrorFile(ByVal FileName As String, ByVal F As Long) As Boolean
    On Error Resume Next
    Open FileName For Append As #F
    TryOpenErrorFile = (Err.Number = 0)
End Function

Private Function GetContext() As Object
    'If this function doesn't compile, then do one of the following
    ' 1) Add reference to "COM+ Services Type Library" in Project|References box.
    ' 2) If you are not using COM+ in your project then add definition
    '       NOCOMPLUS=1 to "Conditional compilation arguments" box on Make page of
    '       Project Properties dialog.
#If H_NOCOMPLUS = 1 Then
    Set GetContext = Nothing
#Else
    Set GetContext = GetObjectContext
#End If
End Function
' -- Script to create table and sp for error logging
'CREATE TABLE [dbo].[tblErrorLog] (
'    [ErrorID] [int] IDENTITY (1, 1) NOT NULL ,  [DateCreated] [datetime] NOT NULL ,
'    [AppID] [int] NULL ,  [ErrorReport] [Text]
') ON [PRIMARY]
'GO
'CREATE PROCEDURE dbo.spErrorLogInsert
'    @AppID INT, @ErrorReport Text
'AS
'INSERT INTO tblErrorLog (DateCreated, AppID, ErrorReport)
'VALUES  (GetDate(),@AppID, @ErrorReport)
'GO
'GRANT EXECUTE ON spErrorLogInsert to public
'GO
'-- End of Script
